#!/usr/bin/env python3

"""We are assuming that all input NetCDF files are in the same
directory and that the time coordinate is in days since 1950-1-1."""

import argparse
import datetime
import shutil
import nco
import os
from os import path
import subprocess
import shapefile
import sys
from dateutil import parser
import netCDF4
import time
import csv

argparser = argparse.ArgumentParser()
argparser.add_argument("first_file",
                       help = "NetCDF file containing SSH and velocity at a "
                       "single date")
argparser.add_argument("--template",
                       help = "template of basename with date format, only "
                       "used for several dates")
argparser.add_argument("-l", "--last_date", help = "%%Y-%%m-%%d")
argparser.add_argument("-b", "--bbox", nargs=4, type = float,
                       metavar = ("xmin", "xmax", "ymin", "ymax"))
args = argparser.parse_args()

if not os.access("main_nml.txt", os.R_OK):
    sys.exit("main_nml.txt not found or not readable")

inst_eddies_exe = "@CMAKE_CURRENT_BINARY_DIR@/inst_eddies"

if not os.access(inst_eddies_exe, os.X_OK):
    sys.exit(inst_eddies_exe + " not found or not executable")

# Get the first date from the time coordinate in the NetCDF file:
with netCDF4.Dataset(args.first_file) as f:
    my_date = f["time"][:].item()

my_date = round(my_date)
my_date = datetime.date(1950, 1, 1) + datetime.timedelta(my_date)
#--

dirname = path.dirname(args.first_file)
nc_file = args.first_file
nco_instance = nco.Nco()

if args.last_date:
    last_date = parser.parse(args.last_date).date()
    
    if last_date > my_date and not args.template:
        sys.exit("template option required for several dates")
else:
    last_date = my_date

for orient in ["cyclo", "anti"]:
    if os.access(f"SHPC_{orient}_all_dates", os.F_OK):
        shutil.rmtree(f"SHPC_{orient}_all_dates")

perf_report = open("perf_report.csv", "w", newline='')
writer = csv.writer(perf_report, lineterminator = "\n")
writer.writerow(["date", "elapsed NCO", "elapsed Fortran", "elapsed cat"])
        
while True:
    print("inst_eddies.py: Date:", my_date)

    if os.access(nc_file, os.F_OK):
        my_pc = time.perf_counter()
        
        if args.bbox:
            xmin, xmax, ymin, ymax = args.bbox
            nco_instance.ncks(nc_file, output = "cropped.nc",
                              options = [f"--dimension=longitude,{xmin},{xmax}",
                                         f"--dimension=latitude,{ymin},{ymax}"])
            nc_file = "cropped.nc"
            
        nco_instance.ncpdq(nc_file, output = "unpacked.nc",
                           options = ["--unpack"])
        nc_file = "unpacked.nc"
        os.symlink(nc_file, "h.nc")
        os.symlink(nc_file, "uv.nc")
        os.mkdir("SHPC_cyclo")
        os.mkdir("SHPC_anti")
        elapsed_NCO = time.perf_counter() - my_pc
        
        with open("main_nml.txt") as nml_file:
            my_pc = time.perf_counter()
            subprocess.run(inst_eddies_exe, stdin = nml_file, check = True)
            elapsed_Fortran = time.perf_counter() - my_pc

        os.remove("h.nc")
        os.remove("uv.nc")
    else:
        print("inst_eddies.py: Missing file:", nc_file)
        elapsed_NCO, elapsed_Fortran = - 1, - 1

    my_pc = time.perf_counter()
    
    for orient in ["cyclo", "anti"]:
        if os.access(f"SHPC_{orient}_all_dates", os.F_OK):
            if os.access(f"SHPC_{orient}", os.F_OK):
                for filename in ["extremum", "max_speed_contour",
                                 "outermost_contour"]:
                    from_file = path.join(f"SHPC_{orient}", filename)
                    to_file = path.join(f"SHPC_{orient}_all_dates", filename)

                    for command in ["dbfcat", "shpcat"]:
                        subprocess.run([command, from_file, to_file],
                                       check = True)

                shutil.rmtree(f"SHPC_{orient}")

            with open(f"SHPC_{orient}_all_dates/ishape_last.txt", "a") as \
                 ishape_last, \
                 shapefile.Reader(f"SHPC_{orient}_all_dates/extremum") as shpr:
                n_shapes = len(shpr)
                ishape_last.write(str(n_shapes - 1) + "\n")
        else:
            if os.access(f"SHPC_{orient}", os.F_OK):
                os.rename(f"SHPC_{orient}", f"SHPC_{orient}_all_dates")

    elapsed_cat = time.perf_counter() - my_pc
    writer.writerow([my_date, elapsed_NCO, elapsed_Fortran, elapsed_cat])
    my_date += datetime.timedelta(1)
    if my_date > last_date: break
    basename = my_date.strftime(args.template)
    nc_file = path.join(dirname, basename)

perf_report.close()
os.remove("unpacked.nc")
if args.bbox: os.remove("cropped.nc")
